unit UClassesGestionEntreprise;

interface

uses Classes, StdCtrls;

Type
         {dclaration forward - annoncer que l'on aura  implmenter des filiales plus loin}
         TFiliale = class;    

         {classe gestion de l'entreprise Duchmol}
         TEntreprise = Class(TObject)
                       private
                       FNomEntreprise: String; //contient le nom de l'entreprise, fix une fois pour toute
                       FLstFiliales: TStringList;//liste des filiales
                       procedure ViderFiliales;//supprimer toutes les filiales de l'entreprise
                       public
                       //constructeur & destructeur
                       constructor Create;
                       destructor  Destroy; override;
                       //gestion des filliales
                       procedure AjouterFiliale(Filiale: TFiliale);//sans la dclaration forward, cette procdure ne peut pas xister
                       function  FilialeExiste(Filiale: TFiliale): Boolean;//permet de savoir si une filiale du mme nom existe dj
                       Function  RecupererFiliale(i: Integer): TFiliale;//rcuprer la filiale numro i
                       procedure DetruireFiliale(i: Integer);//dtruire la filliale numro i
                       //gestion de l'affichage
                       procedure RafraichirAffichage(LstBox: TListBox);
                       //proprits
                       property NomEntreprise: String Read FNomEntreprise;
                       End;

         //dclaration forward pour prparer TFiliale  la gestion des employs
         TEmploye = class;

         {classe gestion d'une filiale}
         TFiliale = Class (TObject)
                    private
                    FLocalisation: String;
                    FTauxRemuneration: Double;
                    FLstEmployes: TStringList;
                    procedure ViderEmployes;//se dfaire de tous les employs de la filiale
                    function  SetStringTaux: String;
                    public
                    //constructeurs & destructeurs
                    constructor Create;
                    destructor  Destroy; override;
                    //entres-sorties
                    function  OkRecupParametres(sLoc: String; sTaux: String): Boolean;
                    procedure AffichageLstEmployes(LstBox: TListBox);
                    //gestion des employes
                    procedure AjouterEmploye(Employe: TEmploye);//ajouter un employ -- ncessite galement la dclaration forward
                    procedure SeDefaireEmploye(i: Integer);//se dfaire de l'employ numro i
                    //proprits
                    property Localisation: String Read FLocalisation;
                    property TauxRemuneration: Double Read FTauxRemuneration;
                    property stringTauxRemuneration: String Read SetStringTaux;
                    End;

         {classe abstraite employe}
         TEmploye = Class(TObject)
                    private
                    FNom: String;
                    FSalaireBase: Currency;//le type currency est spcialis pour le opration montaires - il se comporte comme un rel double mais il est plus prcis
                    FFiliale: TFiliale;
                    procedure setFiliale(prmFiliale: TFiliale);//affecter une filiale  un employ - prmFiliale pour viter les confusions avec la proprit Filiale
                    protected
                    //calculs
                    function Salaire: Currency; virtual;//important qu'elle soit virtuelle, car elle sera poplymorphe !!!
                    public
                    //rcupration des valeurs
                    function okRecupValeurs(sNom: String; sSalaire: String): Boolean;//c'est une mthode statique !!!
                    //affichage
                    function Affichage: String;
                    //proprits
                    property Nom: String Read FNom;
                    property Filiale: TFiliale Read FFiliale Write setFiliale;
                    End;

         {Employ de bureau}
         TEmployeBureau = class(TEmploye)
                          private
                          FAnciennete: Integer; //anciennet au sein de l'entreprise
                          protected
                          //calculs
                          function Salaire: Currency;override;//ici, surcharge dynamique
                          public
                          //rcupration des valeurs
                          function okRecupValeurs(sNom: String; sSalaire: String; sAnciennete: String):boolean;//attention, ce n'est pas une surchage dynamique mais statique, l'en-tte de la fonction n'est pas la mme, pas de polymorphisme possible ici
                          //proprits
                          property Anciennete: Integer Read FAnciennete;
                          End;

         {Cadre}
         TCadre = Class(TEmployeBureau)
                  private
                  FPrimeMensuelle: Currency;
                  protected
                  //calculs
                  function Salaire: Currency;override;//ici, surcharge dynamique
                  public
                  //rcupration des valeurs
                  function okRecupValeurs(sNom: String; sSalaire: String; sAnciennete: String; sPrime: String):boolean;//attention, ce n'est pas une surchage dynamique mais statique, l'en-tte de la fonction n'est pas la mme, pas de polymorphisme possible ici
                  //proprits
                  property PrimeMensuelle: Currency Read FPrimeMensuelle;
                  End;

         {commercial}
         TCommercial = class(TEmploye)
                       private
                       FCa: Currency;
                       protected
                       //calculs
                       function Salaire: Currency;override;//ici, surcharge dynamique
                       public
                       //rcupration des valeurs
                       function okRecupValeurs(sNom: String; sSalaire: String; sCa: String): Boolean;//c'est une mthode statique !!!
                       //proprits
                       property Ca: Currency Read FCa;
                       end;


//variable globale - instance de l'entreprise
var Entreprise: TEntreprise;

implementation

Uses SysUtils;

{ TEntreprise }

procedure TEntreprise.AjouterFiliale(Filiale: TFiliale);
begin
 FLstFiliales.AddObject(Filiale.Localisation,Filiale);
end;

constructor TEntreprise.Create;
begin
 inherited Create;
 FNomEntreprise:= 'Duchmol';
 FLstFiliales:= TStringList.Create;
end;

destructor TEntreprise.Destroy;
begin
  ViderFiliales;//supprimer toutes les filiales
  FLstFiliales.Free;//supprimer le gestionnaire de la liste
  inherited Destroy;
end;

procedure TEntreprise.DetruireFiliale(i: Integer);
Var Filiale: TFiliale;
begin
 //toujours protger l'accs  la liste pour viter les erreurs
 if (i>=0) and (i<FLstFiliales.Count)
  Then
   Begin
    Filiale:= FLstFiliales.Objects[i] As TFiliale;//accder  l'objet
    FLstFiliales.Delete(i);//supprimer la rfrence dans la liste
    Filiale.Free;//supprimer la filiale elle-mme
   End;
end;

function TEntreprise.FilialeExiste(Filiale: TFiliale): Boolean;
begin
 If (FLstFiliales.IndexOf(Filiale.Localisation)>=0)
  Then FilialeExiste:= True
  Else FilialeExiste:= False;
end;

procedure TEntreprise.RafraichirAffichage(LstBox: TListBox);
begin
 LstBox.Items.Assign(FLstFiliales);//copie directe, quivalent d'une boucle
 //code alternatif - copie une  une des filiales
 //Var i: Integer;
 //LstBox.Items.Clear;
 //for i:= 0 To FLstFiliales.Count-1 Do
 //Begin
 // Filiale:= FLstFiliales.Objects[i] As TFiliale;
 // LstBox.Items.AddObject(Filiale.Localisation,Filiale);
 //End;
 //cette deuxime solution est quand un petit peu plus longue par rapport  la premire !!!
end;

function TEntreprise.RecupererFiliale(i: Integer): TFiliale;
begin
 If (i>=0) and (i<FLstFiliales.Count)
  Then RecupererFiliale:= FLstFiliales.Objects[i] As TFiliale
  Else RecupererFiliale:= NIL;//l'objet pointe ur rien du tout -- constante delphi !!!
end;

procedure TEntreprise.ViderFiliales;
var i: Integer;
begin
 //mieux vaut partir de la fin de la liste - a vite les dcalages successifs
 For i:= FLstFiliales.Count-1 DownTo 0 Do
  DetruireFiliale(i);
end;

{ TFiliale }

procedure TFiliale.AffichageLstEmployes(LstBox: TListBox);
begin
 LstBox.Items.Assign(FLstEmployes);//toujours aussi sympathique cette mthode
end;

procedure TFiliale.AjouterEmploye(Employe: TEmploye);
begin
 FLstEmployes.AddObject(Employe.Affichage,Employe);
end;

constructor TFiliale.Create;
begin
 inherited Create;
 FLstEmployes:= TStringList.Create;
end;

destructor TFiliale.Destroy;
begin
  ViderEmployes;//c'est triste mais c'est comme a
  FLstEmployes.Free;
  inherited Destroy;
end;

function TFiliale.OkRecupParametres(sLoc, sTaux: String): Boolean;
Var Ok: Boolean;
begin
 //initialisation - pas d'erreur a priori
 Ok:= True;
 If (sLoc='') //pas de nom introduit
  Then Ok:= FALSE
  Else
   Begin
    FLocalisation:= sLoc;
    TRY
     FTauxRemuneration:= StrToFloat(sTaux);
    EXCEPT
     Ok:= False;//en cas d'erreur, on n'est plus d'accord
    END;
   End;
 //renvoyer le rsultat
 OkRecupParametres:= Ok;
end;

procedure TFiliale.SeDefaireEmploye(i: Integer);
Var Employe: TEmploye;
begin
 //toujours vrifier les tendues
 If (i>=0) and (i<FLstEmployes.Count)
  Then
   Begin
    Employe:= FLstEmployes.Objects[i] As TEmploye;
    FLstEmployes.Delete(i);
    Employe.Free;
   End;
end;

function TFiliale.SetStringTaux: String;
begin
 SetStringTaux:= FloatToStr(FTauxRemuneration);
end;

procedure TFiliale.ViderEmployes;
var i: Integer;
begin
 //mieux vaut partir de la fin de la liste - a vite les dcalages successifs
 For i:= FLstEmployes.Count-1 DownTo 0 Do
  SeDefaireEmploye(i);
end;

{ TEmploye }

function TEmploye.Affichage: String;
Var sTemp: String;
begin
 //le nom , la filiale et le salaire
 sTemp:= FNom+'-';
 If (FFiliale<>Nil) //attention, risque d'erreur si la filiale est mal initialise
  Then sTemp:= sTemp+FFiliale.Localisation+'-';
 sTemp:= sTemp+FloatToStr(Salaire);
 //renvoyer le tout
 Affichage:= sTemp;
end;

function TEmploye.okRecupValeurs(sNom, sSalaire: String): Boolean;
Var ok: Boolean;
begin
 ok:= TRUE;
 If (sNom='')
  Then ok:= FALSE
  Else
   Begin
    FNom:= sNom;
    TRY
     FSalaireBase:= StrToFloat(sSalaire);
    EXCEPT
     ok:= false;
    END;
   End;
 okRecupValeurs:= ok;
end;

function TEmploye.Salaire: Currency;
begin
 Salaire:= FSalaireBase;
end;

procedure TEmploye.setFiliale(prmFiliale: TFiliale);
begin
 FFiliale:= prmFiliale;//et non pas Filiale:= prmFiliale qui provequerait un appel rcursif !!!
end;

{ TEmployeBureau }

function TEmployeBureau.okRecupValeurs(sNom, sSalaire, sAnciennete: String): boolean;
Var ok: Boolean;
begin
 ok:= inherited okRecupValeurs(sNom,sSalaire);//norme astuce pour rutiliser le code de la classe anctre ...
 If ok
  Then
   Begin
    TRY
     FAnciennete:= StrToInt(sAnciennete);
    EXCEPT
     ok:= false;
    END;
   End;
 okRecupValeurs:= ok;
end;

function TEmployeBureau.Salaire: Currency;
Var tmpSalaire: Currency;
begin
 //utilisation de l'hritage
 tmpSalaire:= inherited Salaire;
 //si filiale non affecte, risque d'erreur
 If FFiliale<>Nil Then tmpSalaire:= tmpSalaire + FAnciennete*FFiliale.TauxRemuneration;
 //renvoyer le tout
 Salaire:= tmpSalaire;
end;

{ TCadre }

function TCadre.okRecupValeurs(sNom, sSalaire, sAnciennete, sPrime: String): boolean;
Var ok: Boolean;
begin
 ok:= inherited okRecupValeurs(sNom,sSalaire,sAnciennete);
 If ok
  Then
   Begin
    TRY
     FPrimeMensuelle:= StrToFloat(sPrime);
    EXCEPT
     ok:= FALSE;
    END;
   End;
 okRecupValeurs:= ok;
end;

function TCadre.Salaire: Currency;
begin
 Salaire:= inherited Salaire+FPrimeMensuelle;
end;

{ TCommercial }


function TCommercial.okRecupValeurs(sNom, sSalaire, sCa: String): Boolean;
Var ok: Boolean;
begin
 ok:= inherited OkRecupValeurs(sNom,sSalaire);
 If ok
  Then
   Begin
    TRY
     FCa:= StrToFloat(sCa);
    EXCEPT
     ok:= false;
    END;
   End;
 okRecupValeurs:= ok;
end;

function TCommercial.Salaire: Currency;
begin
 Salaire:= inherited Salaire+0.15*FCa;
end;

end.
